/* BGP routing table
   Copyright (C) 1998, 2001 Kunihiro Ishiguro

   This file is part of GNU Zebra.

   GNU Zebra is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   GNU Zebra is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Zebra; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.  */

/* Zebra common header.
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 Kunihiro Ishiguro

   This file is part of GNU Zebra.

   GNU Zebra is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   GNU Zebra is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Zebra; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.  */


/*
 * Prefix structure.
 * Copyright (C) 1998 Kunihiro Ishiguro
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */


/*
 * Prefix related functions.
 * Copyright (C) 1997, 98, 99 Kunihiro Ishiguro
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#ifndef	_DB_TABLE_H
#define	_DB_TABLE_H

#include "db_prefix.h"

/* bgp_table.h */
struct db_table
{
	struct db_node *top;

	void (*remove_fct)(void *);
	unsigned long count;
};

struct db_node
{
	struct prefix p;

	struct db_table *table;
	struct db_node *parent;
	struct db_node *link[2];
#define l_left   link[0]
#define l_right  link[1]
	void *info;
	unsigned int lock;
};

struct db_table *db_table_init(void (*)(void*));			/* create a table */
void db_table_finish(struct db_table *);				/* destroy table */
unsigned long db_table_count(struct db_table *);			/* nodes number */

struct db_node *db_lock_node(struct db_node *node);			/* get the lock */
void db_unlock_node(struct db_node *node);				/* release lock */

struct db_node *db_table_top(struct db_table *);
struct db_node *db_route_next(struct db_node *);
struct db_node *db_route_next_until(struct db_node *, struct db_node *);

struct db_node *db_node_get(struct db_table *, struct prefix *);	/* add a prefix */
void db_node_delete(struct db_node *node);				/* remove node */

void * db_node_set_info(struct db_node *, void *);	/* set node's info, return previous value */
void * db_node_get_info(struct db_node *);	/* get node's info*/

struct db_node *db_node_match(struct db_table *, struct prefix *);	/* best match */
struct db_node *db_node_match_ipv4(struct db_table *, struct in_addr *);/* best match */
#ifdef HAVE_IPV6
struct db_node *db_node_match_ipv6(struct db_table *, struct in6_addr *);/*best match */
#endif /* HAVE_IPV6 */
struct db_node *db_node_match_exact(struct db_table *, struct prefix *); /* exact match */









/*===============================================================================================================================*/
int db_node_get_ro (struct db_table *table, struct prefix *p);

#endif	/* _DB_TABLE_H */
