
/* BGP routing table
   Copyright (C) 1998, 2001 Kunihiro Ishiguro

   This file is part of GNU Zebra.

   GNU Zebra is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   GNU Zebra is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Zebra; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.  */

/* Zebra common header.
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 Kunihiro Ishiguro

   This file is part of GNU Zebra.

   GNU Zebra is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   GNU Zebra is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Zebra; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.  */


/*
 * Prefix structure.
 * Copyright (C) 1998 Kunihiro Ishiguro
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */


/*
 * Prefix related functions.
 * Copyright (C) 1997, 98, 99 Kunihiro Ishiguro
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */
#include "lib.h"
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>

int port;
struct db_table * table;
void rem(void * e){
	free(e);
}

void server(){
	int sockfd,n;
	struct sockaddr_in servaddr,cliaddr;
	socklen_t len;
	char mesg[1000];

	sockfd=socket(AF_INET,SOCK_DGRAM,0);

	bzero(&servaddr,sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_addr.s_addr=htonl(INADDR_ANY);
	servaddr.sin_port=htons(port);
	bind(sockfd,(struct sockaddr *)&servaddr,sizeof(servaddr));

	char buf[512];
	char buf2[512];
	struct prefix p;
	struct db_node * rn;
	for (;;)
	{
		len = sizeof(cliaddr);
		bzero(mesg, 1000);
		n = recvfrom(sockfd,mesg,1000,0,(struct sockaddr *)&cliaddr,&len);
		
		str2prefix(mesg, &p);
		rn = db_node_match(table, &p);
		bzero(buf, 512);
		bzero(buf2, 512);
		if(rn){
			void * info;
			inet_ntop(rn->p.family, (void *)&rn->p.u.prefix, buf, 512);
			info = db_node_get_info(rn);

			sprintf(buf2, "%s/%d", buf, rn->p.prefixlen);

			db_unlock_node(rn);
		}
		else{
			sprintf(buf2, "0.0.0.0/0");
		}

		sendto(sockfd,buf2,512,0,(struct sockaddr *)&cliaddr,sizeof(cliaddr));
	}
}

int load(char * filename){
	FILE *file = fopen ( filename, "r" );	
	if(!file){
		perror(filename);
		return (FALSE);
	}

	char line [ 128 ]; /* or other suitable maximum line size */

	while ( fgets ( line, sizeof line, file ) != NULL ) /* read a line */
	{
		line[strlen(line)-1] = '\0';
		struct prefix p1;
		struct db_node * rn;
		void * info;

		str2prefix (line, &p1);
		apply_mask(&p1);
		rn = db_node_get(table, &p1);

		info = calloc(1, sizeof(char));
		*(int*)(info) = 1;
		db_node_set_info(rn, info);
	}
	fclose ( file );
	return (TRUE);
}

int
main (int argc, char **argv)
{

	table = db_table_init(&rem);

	if(argc != 3){
		fprintf(stderr, "usage: %s file port_number\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	if(!load(argv[1])){
		exit(EXIT_FAILURE);
	}
	port = atoi(argv[2]);
	printf("loaded\n");
	server();

	db_table_finish(table);
	return EXIT_SUCCESS;
}
